/*

This file is (C) Copyright 2002 Erik S. Johansen <einride@einride.org>, all 
rights reserved.

If you work for or represent an organisation, agency or corporation that is 
wholly or partially funded by a government -OR- you are in any way associated 
with the company Cyber Operations, the following licence applies.

You may not use this file or any derivatives of it for any purpose whatsoever,
including reading beyond this comment, redistribution or compiling, without 
explicit written permission from the copyright holder. 

If you do not work for or represent an organisation, agency or corporation 
that is wholly or partially funded by a government -AND- you are not in any 
way associated with the company Cyber Operations, the GNU General Public 
License, as described in the accompanying file GPL, applies.

*/
#ifndef _IPC_H
#define _IPC_H

typedef struct {
  int type;
  int fd1, fd2;
  int flags;
  char id[100];
  char * intbuf;
  int intbufs, intbufp;
} ipc_struct;


/* Create an IPC channel */
ipc_struct * ipc_create();

/* Attach to an existing IPC channel */
ipc_struct * ipc_attach(char * id);

/* Close an IPC channel endpoint */
void ipc_close(ipc_struct * ipc);

/* Read from an IPC channel.
   0  = No data pending to be read, or no connection
   <0 = Error
   >0 = Number of bytes read
*/
int ipc_read(ipc_struct * ipc, char * buf, int bufsize);

/* Write to an IPC channel.
   0  = No connection established, can't send
   <0 = Error
   >0 = Number of bytes sent
*/
int ipc_write(ipc_struct * ipc, char * buf, int bufsize);

/* Read a data block, expect a length int then data
   0  = No data pending to be read, or no connection
   <0 = Error
   >0 = Data block successfully read
*/
int ipc_readblock(ipc_struct * ipc, void * data, int * datasize);

/* Write a data block prefixed by a length int 
   0 = No connection established, can't send
   <0 = Error
   >0 = Data block successfully sent
*/
int ipc_sendblock(ipc_struct * ipc, void * data, int datasize);
#endif











