/*

This file is (C) Copyright 2002 Erik S. Johansen <einride@einride.org>, all 
rights reserved.

If you work for or represent an organisation, agency or corporation that is 
wholly or partially funded by a government -OR- you are in any way associated 
with the company Cyber Operations, the following licence applies.

You may not use this file or any derivatives of it for any purpose whatsoever,
including reading beyond this comment, redistribution or compiling, without 
explicit written permission from the copyright holder. 

If you do not work for or represent an organisation, agency or corporation 
that is wholly or partially funded by a government -AND- you are not in any 
way associated with the company Cyber Operations, the GNU General Public 
License, as described in the accompanying file GPL, applies.

*/
#ifndef _OPCODE_H
#define _OPCODE_H

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

/* struct to store opcodes for one named instruction */
struct instruction {
  char * name;
  char * opcode;
};

typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned int DWORD;

/* 
 * the opcode field is read two chars a piece. If these
 * start with a ?, the second char defines an expected 
 * argument to the code_addins function, if not the
 * chars are treated as a hex opcode 
 * Argument specs:
 *  1 - BYTE
 *  2 - WORD
 *  4 - DWORD
 *  R - Relative offset - string naming a label in the opcode list
 *  A - Absolute offset - string naming a label in the opcode list
 */

extern struct instruction instructions[];


/* Opaque types used for all opcode manipulation functions */
struct code;
typedef struct code * pcode;

/* create a new opcode collection for use with code_* functions */
pcode code_create();

/* Cleanup an opcode collection */
void code_kill(pcode pc);

/* Append an instruction to an opcode collection */
void code_addins(pcode pc, char * label, char * insname, ...);

/* Append data to an opcode collection */
void code_addraw(pcode pc, char * label, void * data, int datalen);

/* Append an immediate value to an opcode collection */
void code_addbyte(pcode pc, char * label, BYTE v);
void code_addword(pcode pc, char * label, WORD v);
void code_adddword(pcode pc, char * label, DWORD v);

void code_addabsref(pcode pc, char * label, char * reflabel);

/* Calculate the buffer size for an opcode collection */
int code_size(pcode pc);

/* Write opcodes adjusted for specified base address to buffer */
void code_write(pcode pc, DWORD base, void * buf);

#endif











