/*

This file is (C) Copyright 2002 Erik S. Johansen <einride@einride.org>, all 
rights reserved.

If you work for or represent an organisation, agency or corporation that is 
wholly or partially funded by a government -OR- you are in any way associated 
with the company Cyber Operations, the following licence applies.

You may not use this file or any derivatives of it for any purpose whatsoever,
including reading beyond this comment, redistribution or compiling, without 
explicit written permission from the copyright holder. 

If you do not work for or represent an organisation, agency or corporation 
that is wholly or partially funded by a government -AND- you are not in any 
way associated with the company Cyber Operations, the GNU General Public 
License, as described in the accompanying file GPL, applies.

*/
#include "options.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

void opt_listoptions(struct option * options) {
  int i, n, maxllen=0, maxalen=0;
  char * ch, ln[1024];
  for (i=0; options[i].longname; i++) {
    n = strlen(options[i].longname);
    if (n>maxllen)
      maxllen = n;
    n = strlen(options[i].argdesc);
    if (n>maxalen)
      maxalen = n;
  }
  for (i=0; options[i].longname; i++) {
    sprintf(ln, "  -%c, --%s", options[i].shortname, options[i].longname);
    ch = ln + strlen(ln);
    while (ch - ln < (maxllen+9) )
      *ch++=' ';
    sprintf(ch, "%s", options[i].argdesc);
    ch += strlen(ch);
    while (ch - ln < (maxllen+maxalen+11) )
      *ch++=' ';
    strcpy(ch, options[i].desc);
    printf("%s\n", ln);
  }
}


void opt_usage(char * argv[], struct option * options, usageproc preopt_usage, usageproc postopt_usage) {
  if (preopt_usage)
    preopt_usage(argv[0]);
  opt_listoptions(options);
  if (postopt_usage)
    postopt_usage(argv[0]);
}

int opt_parse(int argc, char * argv[],
              struct option * options,
              optionproc optproc,
              usageproc preopt_usage, usageproc postopt_usage) 
{
  int argndx = 1, curopt, nextarg, i;
  char wrkbuf[512], *wrk;
 
  while (argndx<argc) {
    strncpy(wrkbuf, argv[argndx], sizeof(wrkbuf));
    wrkbuf[sizeof(wrkbuf)-1] = 0;
    wrk = wrkbuf;
    if (*wrk++ != '-') {
      fprintf(stderr, "Invalid argument %s\n", argv[argndx]);
      opt_usage(argv, options, preopt_usage, postopt_usage);
      return 0;
    }
    curopt = -1;
    if (*wrk == '-') {
      wrk++;
      for (i=0;options[i].longname;i++) {
	if (!strcmp(options[i].longname, wrk)) {
	  curopt = i;
	  break;
	}
      }
    } else {
      for (i=0;options[i].longname;i++) {
	if (options[i].shortname == *wrk) {
	  curopt = i;
	  break;
	}
      }
    }
    if (curopt == -1) {
      fprintf(stderr, "Invalid argument %s\n", argv[argndx]);
      opt_usage(argv, options, preopt_usage, postopt_usage);
      return 0;
    }
    nextarg = argndx + 1;
    while (nextarg<argc && (argv[nextarg][0] != '-'))
      nextarg++;
    if (!optproc(curopt, (nextarg - argndx - 1), &argv[argndx+1])) {
      fprintf(stderr, "Invalid argument parameters for %s\n", argv[argndx]);
      opt_usage(argv, options, preopt_usage, postopt_usage);
      return 0;      
    }
    argndx=nextarg;
  }
  return 1;

}




