/*

This file is (C) Copyright 2002 Erik S. Johansen <einride@einride.org>, all 
rights reserved.

If you work for or represent an organisation, agency or corporation that is 
wholly or partially funded by a government -OR- you are in any way associated 
with the company Cyber Operations, the following licence applies.

You may not use this file or any derivatives of it for any purpose whatsoever,
including reading beyond this comment, redistribution or compiling, without 
explicit written permission from the copyright holder. 

If you do not work for or represent an organisation, agency or corporation 
that is wholly or partially funded by a government -AND- you are not in any 
way associated with the company Cyber Operations, the GNU General Public 
License, as described in the accompanying file GPL, applies.

*/
#ifndef _UTIL_H
#define _UTIL_H
#include "sysinc.h"
#include "ipc.h"
#include "trace.h"

#define DIR_IN                 1 // From external to target
#define DIR_OUT                2 // From target to external

#define CMD_EXIT               1
#define CMD_FDLIST             2
#define CMD_CLOSE              3
#define CMD_SEND               4
#define CMD_WATCH              5
#define CMD_STOPWATCH          6

#define RESP_FDLIST            1
#define RESP_DATA              2
#define RESP_CLOSE             3

#define BACKLOG_BUFFER_SIZE    (80*25*2*4)

typedef struct fdrelay_struct {
  struct fdrelay_struct * next;
  int trg_FD;
  int trg_SavedFD;
  ipc_struct * ipc;
} fdrelay;


#define ACTION_LOG       0x01
#define ACTION_TAKE      0x02
#define ACTION_INSERTIN  0x04
#define ACTION_INSERTOUT 0x08
#define ACTION_SNOOP     0x10

typedef struct task_struct {
  struct task_struct * next;
  int fd;
  int actions;
  FILE * logfile;
  void * insert_data[2];
  int insert_datalen[2];
  char wdata[BACKLOG_BUFFER_SIZE];
  int wdatastart, wdataend;
} task;


#define FDTYPE_SOCKET 1
typedef struct fdinfo_struct {
  int fd;
  int flags;
  int owner;
  int iosig;
  int type;
  char sockname[sizeof(struct sockaddr_un)];
  char peername[sizeof(struct sockaddr_un)];
  struct fdinfo_struct * next;
} fdinfo;

typedef struct mapentry_struct {
  unsigned long vm_start;
  unsigned long vm_end;
  int vm_read;
  int vm_write;
  int vm_exec;
  int vm_shared;
  unsigned long vm_pgoff;
  char devname[16];
  unsigned long inode;
  char file[1024];
  struct mapentry_struct * next;
  int userdata;
} mapentry;

struct app_arg_struct {
  pid_t pid;
  int verbose;
  int expire;
  int gui;
  int watch;
  int memdump;
  task * tasks;
};

struct app_glob_struct {
  pprocess process;
  char binname[1024];
  fdinfo * fdlist;
  int fdcount;
  mapentry * map;  
  unsigned long insertionpoint;
  struct registers savedregs;
  char savedcode[8192];
  fdrelay * relay;
  ipc_struct * ipc;
  int done;
  int currentfd;
  int commandmode;
  int blockint;
  //  int blockstd;
  char cmdstr[20];
  WINDOW * datawin;
  WINDOW * statuswin;
  WINDOW * menuwin;
  void (*savedsigint)(int);
  unsigned long started;
  struct termios savedtio;
  int interactivedir;
  int echoinput;
  int dumpdone;
};

struct child_glob_struct {
  fdinfo * fdlist;
  int fdcount;
  ipc_struct * ipc;
  int watch[256];
  int watchcount;
};


task * util_addtask();
void util_deltask(task * task);
task * util_findtask(int fd);
fdrelay * util_addrelay();
void util_delrelay(fdrelay * relay);
fdrelay * util_findrelay();
int util_parsequotes(char * src, int * len);
int util_addtask_log(char * what, char * file);
int util_addtask_insert(char * what, char * data, int direction);
int util_addtask_take(char * what);
int util_addtask_snoop(char * what);

int util_setoption(int ndx, int argc, char * argv[]);
void util_preoptusage(char * bin);
void util_postoptusage(char * bin);
int util_sendchildcmd(int cmd, void * data, int datalen);
int util_getchildresp(int * resp, void ** data, int * datasize);

char * util_getbinname(char * arg0);
mapentry * util_makemapentry(unsigned int pid);
int util_getprocessbinname(unsigned int pid, int removepath, char * buf, int bufsize);


void util_cyclicbufferappend(char * buffer, int buffersize, int * buffirst, int * bufnext,
			    char * data, int datalen);

void ui_appenddata(char * data, int datalen);
#endif

